==> ejc-tax-impl-3.0.0-20210308.090929-89-sources.jar: com/ejianc/business/tax/service/impl/InvoiceReceiveServiceImpl.java <==
package com.ejianc.business.tax.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ejianc.business.equipment.api.IContractInvoiceReceiveApi;
import com.ejianc.business.income.vo.ContractVo;
import com.ejianc.business.material.api.IMaterialContractApi;
import com.ejianc.business.sub.api.ISubContractInvoiceReceiveApi;
import com.ejianc.business.tax.bean.InvoiceReceiveEntity;
import com.ejianc.business.tax.bean.InvoiceReceiveEntity;
import com.ejianc.business.tax.bean.InvoiceReceiveEntity;
import com.ejianc.business.tax.mapper.InvoiceReceiveMapper;
import com.ejianc.business.tax.mapper.InvoiceReceiveMapper;
import com.ejianc.business.tax.service.IInvoiceReceiveService;
import com.ejianc.business.tax.vo.*;
import com.ejianc.foundation.orgcenter.api.IOrgApi;
import com.ejianc.foundation.orgcenter.vo.OrgVO;
import com.ejianc.foundation.support.api.IBillCodeApi;
import com.ejianc.framework.cache.utils.RedisTool;
import com.ejianc.framework.core.context.InvocationInfoProxy;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.response.Parameter;
import com.ejianc.framework.skeleton.fields.service.ICommenQueryFieldsService;
import com.ejianc.framework.skeleton.template.BaseServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.math.BigDecimal.ROUND_HALF_DOWN;

/**
 * <p>
 * 有合同开票 服务实现类
 * </p>
 *
 * @author yqls
 * @since 2020-06-04
 */
@Service("InvoiceReceiveService")
public class InvoiceReceiveServiceImpl extends BaseServiceImpl<InvoiceReceiveMapper, InvoiceReceiveEntity> implements IInvoiceReceiveService {
    private static final String TAX_INVOICE_RECEIVE_BILL_CODE = "TAX_RECEIVE";
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private IBillCodeApi billCodeApi;
    @Autowired
    private InvoiceReceiveMapper invoiceReceiveMapper;
    @Autowired
    private IOrgApi orgApi;
    @Autowired
    private JedisPool jedisPool;
    @Autowired
    private ICommenQueryFieldsService commenQueryFieldsService;
    @Override
    public CommonResponse<InvoiceReceiveVO> saveOrUpdate(InvoiceReceiveVO invoiceReceiveVo) {

        //保存时校验合同version是否一致
        if(invoiceReceiveVo.getContractVersion()!=null&&invoiceReceiveVo.getContractVersion()!=0) {
            Jedis jedis = jedisPool.getResource();
            boolean locked = false;
            try {
                locked = RedisTool.tryLock(jedis, String.valueOf(invoiceReceiveVo.getContractId()), "saveOrUpdate", 1000);
                logger.info("判断单据单据锁结果------" + locked);
                if (locked) {
                    JSONObject contractEntity = commenQueryFieldsService.queryBillDetail(String.valueOf(invoiceReceiveVo.getContractId()), getBillTypeCode(invoiceReceiveVo.getContractType()));
                    String objVersion = contractEntity.getJSONObject("data").get("version").toString();
                    Integer version =contractEntity.getJSONObject("data").get("version")==null?0:Integer.parseInt(objVersion.substring(0,objVersion.indexOf(".")));
                    Integer conVersion = invoiceReceiveVo.getContractVersion();
                    if (!version.equals(conVersion)) {
                        return CommonResponse.error("该合同已被更新，请刷新后重做！");
                    }
                }else{
                    return CommonResponse.error("出现并发操作,请稍后重试！");
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (locked) {
                    RedisTool.releaseLock(jedis, String.valueOf(invoiceReceiveVo.getContractId()), "saveOrUpdate");
                }
                jedis.close();
            }
        }
        Long tenantId = InvocationInfoProxy.getTenantid();
        InvoiceReceiveEntity entity = null;
        if(StringUtils.isEmpty(invoiceReceiveVo.getBillCode())){
            //新增
            CommonResponse<String> billCode = billCodeApi.getCodeBatchByRuleCode(TAX_INVOICE_RECEIVE_BILL_CODE,tenantId);
            if(billCode.isSuccess()) {
                invoiceReceiveVo.setBillCode(billCode.getData());
            }else{
                throw new BusinessException("网络异常， 编码生成失败， 请稍后再试");
            }
        }
        if(invoiceReceiveVo.getId() != null && invoiceReceiveVo.getId() > 0) {
            //修改  校验合同编号是否重复
            LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
            lambda.eq(InvoiceReceiveEntity::getBillCode, invoiceReceiveVo.getBillCode());
            lambda.eq(InvoiceReceiveEntity::getTenantId, tenantId);
            lambda.ne(InvoiceReceiveEntity::getId, invoiceReceiveVo.getId());
            List<InvoiceReceiveEntity> entities = super.list(lambda);
            if(entities != null && entities.size() > 0) {
                throw new BusinessException("存在相同编码，不允许保存!");
            }
        }else{
            //校验合同编号是否重复
            LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
            lambda.eq(InvoiceReceiveEntity::getTenantId, tenantId);
            lambda.eq(InvoiceReceiveEntity::getBillCode, invoiceReceiveVo.getBillCode());
            List<InvoiceReceiveEntity> entities = super.list(lambda);
            if(entities != null && entities.size() > 0) {
                throw new BusinessException("存在相同编码，不允许保存!");
            }
        }
        entity = BeanMapper.map(invoiceReceiveVo, InvoiceReceiveEntity.class);
        entity.setFlag(0);
        super.saveOrUpdate(entity,false);

        return CommonResponse.success(BeanMapper.map(entity, InvoiceReceiveVO.class));
    }
    @Override
    public InvoiceReceiveVO queryDetail(Long id) {
        InvoiceReceiveEntity entity = invoiceReceiveMapper.selectById(id);
        if(entity != null) {
            InvoiceReceiveVO invoiceReceiveVo = BeanMapper.map(entity, InvoiceReceiveVO.class);
            return invoiceReceiveVo;
        }
        return null;
    }

    @Override
    public void deleteInvoiceReceive(List<InvoiceReceiveVO> vos) {
        super.removeByIds(vos.stream().map(InvoiceReceiveVO::getId).collect(Collectors.toList()),false);
    }

    @Override
    public BigDecimal getReceiveMny(Long contractId, Long orgId) {
        BigDecimal receiveMny = invoiceReceiveMapper.getReceiveMny(contractId,orgId);
        return receiveMny;
    }

    @Override
    public InvoiceReceiveRecordVO getInvoiceReceiveRecord(Long id) {

        InvoiceReceiveRecordVO invoiceReceiveRecordVO = new InvoiceReceiveRecordVO();
        invoiceReceiveRecordVO.setContractId(id);
        List<Long> longList = orgApi.findChildrenByParentId(InvocationInfoProxy.getOrgId()).getData().stream().map(OrgVO::getId).collect(Collectors.toList());
        LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
        lambda.eq(InvoiceReceiveEntity::getContractId,id);
        lambda.in(InvoiceReceiveEntity::getBillState, BillStateEnum.PASSED_STATE.getBillStateCode(),BillStateEnum.COMMITED_STATE.getBillStateCode());
        lambda.in(InvoiceReceiveEntity::getOrgId, longList);
        lambda.orderByDesc(InvoiceReceiveEntity::getCreateTime);
        List<InvoiceReceiveEntity> entities = super.list(lambda);
        if(CollectionUtils.isNotEmpty(entities)){
            List<InvoiceReceiveVO> list = BeanMapper.mapList(entities, InvoiceReceiveVO.class);
            BigDecimal invoiceMny = BigDecimal.ZERO;//发票金额（不含税）
            BigDecimal invoiceTaxMny = BigDecimal.ZERO;//发票金额（含税）
            BigDecimal taxMny = BigDecimal.ZERO;//税金
            for (InvoiceReceiveVO invoiceReceiveVO : list){
                invoiceReceiveVO.setBillStateName(BillStateEnum.getEnumByStateCode(invoiceReceiveVO.getBillState()).getDescription());
                invoiceMny = invoiceMny.add(invoiceReceiveVO.getInvoiceMny());
                invoiceTaxMny = invoiceTaxMny.add(invoiceReceiveVO.getInvoiceTaxMny());
                taxMny = taxMny.add(invoiceReceiveVO.getTaxMny());
            }
            invoiceReceiveRecordVO.setInvoiceMny(invoiceMny);
            invoiceReceiveRecordVO.setInvoiceTaxMny(invoiceTaxMny);
            invoiceReceiveRecordVO.setTaxMny(taxMny);
            invoiceReceiveRecordVO.setInvoiceReceiveVOList(list);
        }
        return invoiceReceiveRecordVO;
    }

    @Override
    public CommonResponse<String> updateFlag(List<InvoiceReceiveFlagVO> invoiceReceiveFlagVOS) {
        List<InvoiceReceiveEntity> list = BeanMapper.mapList(invoiceReceiveFlagVOS, InvoiceReceiveEntity.class);
        List<Long> isLongList = new ArrayList<>();
        List<Long> notIslongList = new ArrayList<>();
        for (InvoiceReceiveEntity invoiceReceiveEntity : list){
            if (invoiceReceiveEntity.getFlag().equals(0)){
                notIslongList.add(invoiceReceiveEntity.getId());
            }
            if (invoiceReceiveEntity.getFlag().equals(1)){
                isLongList.add(invoiceReceiveEntity.getId());
            }
        }
        //
        if (isLongList != null && isLongList.size() > 0){
            LambdaUpdateWrapper<InvoiceReceiveEntity> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.in(InvoiceReceiveEntity::getId, isLongList);
            updateWrapper.set(InvoiceReceiveEntity::getFlag, 1);//(1:是，0：否)
            super.update(updateWrapper);
        }
        if (notIslongList != null && notIslongList.size() > 0){
            LambdaUpdateWrapper<InvoiceReceiveEntity> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.in(InvoiceReceiveEntity::getId, notIslongList);
            updateWrapper.set(InvoiceReceiveEntity::getFlag, 0);//(1:是，0：否)
            super.update(updateWrapper);
        }
        for (InvoiceReceiveEntity invoiceReceiveEntity : list){

        }
            return CommonResponse.success("修改收票引用标识成功！");
    }

    @Override
    public Map<String, InvoiceMnyWarnVO> getInvoiceMnyByTenantIds(PubWarnQueryVO pubWarnQueryVO) {
        if (null == pubWarnQueryVO.getContractType()) {
            throw new BusinessException("合同类型不能为空！");
        }
        Map<String, InvoiceMnyWarnVO> map = new HashMap<>();
        List<InvoiceMnyWarnVO> list = baseMapper.getInvoiceMnyByTenantIds(pubWarnQueryVO);
        for (InvoiceMnyWarnVO invoiceMnyWarnVO : list) {
            map.put(invoiceMnyWarnVO.getTenantId() + "@" + invoiceMnyWarnVO.getOrgId() + "@" + invoiceMnyWarnVO.getContractId(), invoiceMnyWarnVO);
        }
        return map;
    }

    public String getBillTypeCode(String contractType){
        if("1".equals(contractType)){//分包合同
            return "BT200529000000001";
        }else if("2".equals(contractType)){//设备采购合同
            return "BT200528000000001";
        }else if("3".equals(contractType)){//设备租赁合同
            return "BT200528000000003";
        }else if("4".equals(contractType)){//物资采购合同
            return "BT200608000000001";
        }else if("5".equals(contractType)){//物资租赁合同
            return "BT200608000000001";
        }
        return "";
    }
}

==> ejc-tax-impl-3.0.0-20210820.114525-132-sources.jar: com/ejianc/business/tax/service/impl/InvoiceReceiveServiceImpl.java <==
package com.ejianc.business.tax.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ejianc.business.tax.bean.InvoiceReceiveEntity;
import com.ejianc.business.tax.mapper.InvoiceReceiveMapper;
import com.ejianc.business.tax.service.IInvoiceReceiveService;
import com.ejianc.business.tax.utils.ComputeUtil;
import com.ejianc.business.tax.vo.*;
import com.ejianc.foundation.orgcenter.api.IOrgApi;
import com.ejianc.foundation.orgcenter.vo.OrgVO;
import com.ejianc.foundation.support.api.IBillCodeApi;
import com.ejianc.framework.cache.utils.RedisTool;
import com.ejianc.framework.core.context.InvocationInfoProxy;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.skeleton.fields.service.ICommenQueryFieldsService;
import com.ejianc.framework.skeleton.template.BaseServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 有合同开票 服务实现类
 * </p>
 *
 * @author yqls
 * @since 2020-06-04
 */
@Service("InvoiceReceiveService")
public class InvoiceReceiveServiceImpl extends BaseServiceImpl<InvoiceReceiveMapper, InvoiceReceiveEntity> implements IInvoiceReceiveService {
    private static final String TAX_INVOICE_RECEIVE_BILL_CODE = "TAX_RECEIVE";
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private IBillCodeApi billCodeApi;
    @Autowired
    private InvoiceReceiveMapper invoiceReceiveMapper;
    @Autowired
    private IOrgApi orgApi;
    @Autowired
    private JedisPool jedisPool;
    @Autowired
    private ICommenQueryFieldsService commenQueryFieldsService;

    @Override
    public CommonResponse<InvoiceReceiveVO> saveOrUpdate(InvoiceReceiveVO invoiceReceiveVo) {

        //保存时校验合同version是否一致
        if (invoiceReceiveVo.getContractVersion() != null && invoiceReceiveVo.getContractVersion() != 0) {
            Jedis jedis = jedisPool.getResource();
            boolean locked = false;
            try {
                locked = RedisTool.tryLock(jedis, String.valueOf(invoiceReceiveVo.getContractId()), "saveOrUpdate", 1000);
                logger.info("判断单据单据锁结果------" + locked);
                if (locked) {
                    JSONObject contractEntity = commenQueryFieldsService.queryBillDetail(String.valueOf(invoiceReceiveVo.getContractId()), getBillTypeCode(invoiceReceiveVo.getContractType()));
                    String objVersion = contractEntity.getJSONObject("data").get("version").toString();
                    Integer version = contractEntity.getJSONObject("data").get("version") == null ? 0 : Integer.parseInt(objVersion.substring(0, objVersion.indexOf(".")));
                    Integer conVersion = invoiceReceiveVo.getContractVersion();
                    if (!version.equals(conVersion)) {
                        return CommonResponse.error("该合同已被更新，请刷新后重做！");
                    }
                } else {
                    return CommonResponse.error("出现并发操作,请稍后重试！");
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (locked) {
                    RedisTool.releaseLock(jedis, String.valueOf(invoiceReceiveVo.getContractId()), "saveOrUpdate");
                }
                jedis.close();
            }
        }
        Long tenantId = InvocationInfoProxy.getTenantid();
        InvoiceReceiveEntity entity = null;
        if (StringUtils.isEmpty(invoiceReceiveVo.getBillCode())) {
            //新增
            CommonResponse<String> billCode = billCodeApi.getCodeBatchByRuleCode(TAX_INVOICE_RECEIVE_BILL_CODE, tenantId);
            if (billCode.isSuccess()) {
                invoiceReceiveVo.setBillCode(billCode.getData());
            } else {
                throw new BusinessException("网络异常， 编码生成失败， 请稍后再试");
            }
        }
        if (invoiceReceiveVo.getId() != null && invoiceReceiveVo.getId() > 0) {
            //修改  校验合同编号是否重复
            LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
            lambda.eq(InvoiceReceiveEntity::getBillCode, invoiceReceiveVo.getBillCode());
            lambda.eq(InvoiceReceiveEntity::getTenantId, tenantId);
            lambda.ne(InvoiceReceiveEntity::getId, invoiceReceiveVo.getId());
            List<InvoiceReceiveEntity> entities = super.list(lambda);
            if (entities != null && entities.size() > 0) {
                throw new BusinessException("存在相同编码，不允许保存!");
            }
        } else {
            //校验合同编号是否重复
            LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
            lambda.eq(InvoiceReceiveEntity::getTenantId, tenantId);
            lambda.eq(InvoiceReceiveEntity::getBillCode, invoiceReceiveVo.getBillCode());
            List<InvoiceReceiveEntity> entities = super.list(lambda);
            if (entities != null && entities.size() > 0) {
                throw new BusinessException("存在相同编码，不允许保存!");
            }
        }
        entity = BeanMapper.map(invoiceReceiveVo, InvoiceReceiveEntity.class);
        entity.setFlag(0);
        super.saveOrUpdate(entity, false);

        return CommonResponse.success(BeanMapper.map(entity, InvoiceReceiveVO.class));
    }

    @Override
    public InvoiceReceiveVO queryDetail(Long id) {
        InvoiceReceiveEntity entity = invoiceReceiveMapper.selectById(id);
        if (entity != null) {
            InvoiceReceiveVO invoiceReceiveVo = BeanMapper.map(entity, InvoiceReceiveVO.class);
            return invoiceReceiveVo;
        }
        return null;
    }

    @Override
    public void deleteInvoiceReceive(List<InvoiceReceiveVO> vos) {
        super.removeByIds(vos.stream().map(InvoiceReceiveVO::getId).collect(Collectors.toList()), false);
    }

    @Override
    public BigDecimal getReceiveMny(Long contractId, Long orgId) {
        BigDecimal receiveMny = invoiceReceiveMapper.getReceiveMny(contractId, orgId);
        return receiveMny;
    }

    @Override
    public InvoiceReceiveRecordVO getInvoiceReceiveRecord(Long id) {

        InvoiceReceiveRecordVO invoiceReceiveRecordVO = new InvoiceReceiveRecordVO();
        invoiceReceiveRecordVO.setContractId(id);
        List<Long> longList = orgApi.findChildrenByParentId(InvocationInfoProxy.getOrgId()).getData().stream().map(OrgVO::getId).collect(Collectors.toList());
        LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
        lambda.eq(InvoiceReceiveEntity::getContractId, id);
        lambda.in(InvoiceReceiveEntity::getBillState, BillStateEnum.PASSED_STATE.getBillStateCode(), BillStateEnum.COMMITED_STATE.getBillStateCode());
        lambda.in(InvoiceReceiveEntity::getOrgId, longList);
        lambda.orderByDesc(InvoiceReceiveEntity::getCreateTime);
        List<InvoiceReceiveEntity> entities = super.list(lambda);
        if (CollectionUtils.isNotEmpty(entities)) {
            List<InvoiceReceiveVO> list = BeanMapper.mapList(entities, InvoiceReceiveVO.class);
            BigDecimal invoiceMny = BigDecimal.ZERO;//发票金额（不含税）
            BigDecimal invoiceTaxMny = BigDecimal.ZERO;//发票金额（含税）
            BigDecimal taxMny = BigDecimal.ZERO;//税金
            for (InvoiceReceiveVO invoiceReceiveVO : list) {
                invoiceReceiveVO.setBillStateName(BillStateEnum.getEnumByStateCode(invoiceReceiveVO.getBillState()).getDescription());
                invoiceMny = invoiceMny.add(invoiceReceiveVO.getInvoiceMny());
                invoiceTaxMny = invoiceTaxMny.add(invoiceReceiveVO.getInvoiceTaxMny());
                taxMny = taxMny.add(invoiceReceiveVO.getTaxMny());
            }
            invoiceReceiveRecordVO.setInvoiceMny(invoiceMny);
            invoiceReceiveRecordVO.setInvoiceTaxMny(invoiceTaxMny);
            invoiceReceiveRecordVO.setTaxMny(taxMny);
            invoiceReceiveRecordVO.setInvoiceReceiveVOList(list);
        }
        return invoiceReceiveRecordVO;
    }

    @Override
    public CommonResponse<String> updateFlag(List<InvoiceReceiveFlagVO> invoiceReceiveFlagVOS) {
        List<InvoiceReceiveEntity> list = BeanMapper.mapList(invoiceReceiveFlagVOS, InvoiceReceiveEntity.class);
        List<Long> isLongList = new ArrayList<>();
        List<Long> notIslongList = new ArrayList<>();
        for (InvoiceReceiveEntity invoiceReceiveEntity : list) {
            if (invoiceReceiveEntity.getFlag().equals(0)) {
                notIslongList.add(invoiceReceiveEntity.getId());
            }
            if (invoiceReceiveEntity.getFlag().equals(1)) {
                isLongList.add(invoiceReceiveEntity.getId());
            }
        }
        //
        if (isLongList != null && isLongList.size() > 0) {
            LambdaUpdateWrapper<InvoiceReceiveEntity> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.in(InvoiceReceiveEntity::getId, isLongList);
            updateWrapper.set(InvoiceReceiveEntity::getFlag, 1);//(1:是，0：否)
            super.update(updateWrapper);
        }
        if (notIslongList != null && notIslongList.size() > 0) {
            LambdaUpdateWrapper<InvoiceReceiveEntity> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.in(InvoiceReceiveEntity::getId, notIslongList);
            updateWrapper.set(InvoiceReceiveEntity::getFlag, 0);//(1:是，0：否)
            super.update(updateWrapper);
        }
        for (InvoiceReceiveEntity invoiceReceiveEntity : list) {

        }
        return CommonResponse.success("修改收票引用标识成功！");
    }

    @Override
    public Map<String, InvoiceMnyWarnVO> getInvoiceMnyByTenantIds(InvoicePubWarnQueryVO pubWarnQueryVO) {
        if (null == pubWarnQueryVO.getContractType()) {
            throw new BusinessException("合同类型不能为空！");
        }
        Map<String, InvoiceMnyWarnVO> map = new HashMap<>();
        List<InvoiceMnyWarnVO> list = baseMapper.getInvoiceMnyByTenantIds(pubWarnQueryVO);
        for (InvoiceMnyWarnVO invoiceMnyWarnVO : list) {
            map.put(invoiceMnyWarnVO.getTenantId() + "@" + invoiceMnyWarnVO.getOrgId() + "@" + invoiceMnyWarnVO.getContractId(), invoiceMnyWarnVO);
        }
        return map;
    }

    @Override
    public List<InvoiceReceiveVO> getTaxReceive(Long projectId) {
        List<Integer> billStatus = new ArrayList<>();
        billStatus.add(1);
        billStatus.add(3);
        LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
        lambda.eq(InvoiceReceiveEntity::getProjectId, projectId);
        lambda.eq(InvoiceReceiveEntity::getDr, 0);
        lambda.in(InvoiceReceiveEntity::getBillState, billStatus);
        List<InvoiceReceiveEntity> entities = super.list(lambda);
        return BeanMapper.mapList(entities, InvoiceReceiveVO.class);
    }

    @Override
    public Map<String, InvoiceMnyWarnVO> getWarnInvoiceMny(InvoicePubWarnQueryVO pubWarnQueryVO) {
        if (null == pubWarnQueryVO.getContractType()) {
            throw new BusinessException("合同类型不能为空！");
        }
        Map<String, InvoiceMnyWarnVO> map = new HashMap<>();
        List<InvoiceMnyWarnVO> list = baseMapper.getWarnInvoiceMny(pubWarnQueryVO);
        for (InvoiceMnyWarnVO invoiceMnyWarnVO : list) {
            map.put(invoiceMnyWarnVO.getTenantId() + "@" + invoiceMnyWarnVO.getOrgId() + "@" + invoiceMnyWarnVO.getContractId(), invoiceMnyWarnVO);
        }
        return map;
    }

    @Override
    public CommonResponse<String> updateUsedInvoiceMny(List<InvoiceReceiveFlagVO> invoiceReceiveFlagVOS) {
        if (CollectionUtils.isNotEmpty(invoiceReceiveFlagVOS)) {
            List<InvoiceReceiveEntity> list = BeanMapper.mapList(invoiceReceiveFlagVOS, InvoiceReceiveEntity.class);
            List<Long> ids = list.stream().map(InvoiceReceiveEntity::getId).collect
                    (Collectors.toList());

            LambdaQueryWrapper<InvoiceReceiveEntity> lambda = Wrappers.<InvoiceReceiveEntity>lambdaQuery();
            lambda.in(InvoiceReceiveEntity::getId, ids);
            List<InvoiceReceiveEntity> entities = super.list(lambda);

            if (CollectionUtils.isNotEmpty(entities)) {
                Map<Long, InvoiceReceiveEntity> maps = entities.stream().collect(Collectors.toMap(InvoiceReceiveEntity::getId, Function.identity(), (key1, key2) -> key2));

                // 校验当钱可用金额是否足以被使用，有可能会有并发和时间差操作
                for (InvoiceReceiveFlagVO i : invoiceReceiveFlagVOS) {
                    Long id = i.getId();
                    InvoiceReceiveEntity e = maps.get(id);
                    if (e != null) {
                        BigDecimal use = ComputeUtil.safeAdd(e.getUsedInvoiceTaxMny(), i.getNowUesdInvoiceTaxMny());
                        //发票金额 小于已占用+ 当前占用
                        if (ComputeUtil.isLessThan(e.getInvoiceTaxMny(), use)) {
                            return CommonResponse.error("操作失败：发票【号码：" + e.getInvoiceNumber() + "】的剩余金额为：" + ComputeUtil.scaleTwo(ComputeUtil.safeSub(e.getInvoiceTaxMny(), e.getUsedInvoiceTaxMny())) + "，请刷新后重新选择！");
                        }
                        i.setNowUesdInvoiceTaxMny(use);
                        i.setFlag(ComputeUtil.equals(e.getInvoiceTaxMny(), use) ? 1 : 0);
                    }
                }
            }
            // 更新发票金额和占用状态
            for (InvoiceReceiveFlagVO i : invoiceReceiveFlagVOS) {
                LambdaUpdateWrapper<InvoiceReceiveEntity> updateWrapper = new LambdaUpdateWrapper<>();
                updateWrapper.eq(InvoiceReceiveEntity::getId, i.getId());
                updateWrapper.set(InvoiceReceiveEntity::getFlag, i.getFlag());//(1:是，0：否)
                updateWrapper.set(InvoiceReceiveEntity::getUsedInvoiceTaxMny, i.getNowUesdInvoiceTaxMny());
                super.update(updateWrapper);
            }
        }

        return CommonResponse.success("修改收票引用标识成功！");
    }

    public String getBillTypeCode(String contractType) {
        if ("1".equals(contractType)) {//分包合同
            return "BT200529000000001";
        } else if ("2".equals(contractType)) {//设备采购合同
            return "BT200528000000001";
        } else if ("3".equals(contractType)) {//设备租赁合同
            return "BT200528000000003";
        } else if ("4".equals(contractType)) {//物资采购合同
            return "BT200608000000001";
        } else if ("5".equals(contractType)) {//物资租赁合同
            return "BT200608000000001";
        }
        return "";
    }
}
